﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class ObstacleCircle : MonoBehaviour
{
    [SerializeField] private GameObject obstacle = null;
    [SerializeField] private int numberOfUnits = 0;
    [SerializeField] private float rotationOffset = 0f;
    [SerializeField] private float[] rotationSpeed = null;
    // 'o' means obstacle, followed by the number of adjacent of them.
    // 'e' means empty or no obstacle, followed by the number of empty units
    // the sum of all numbers in an string should be equal to the numberOfUnits field;
    [SerializeField] private string[] levelEditor = null;

    private float initialRotationY;
    private ArrayList obstacles = new ArrayList();
    private int levelIndex = 0;

    private void Awake()
    {
        initialRotationY = transform.localEulerAngles.y;
    }

    private void Start()
    {
        transform.localEulerAngles += new Vector3(0f, rotationOffset, 0f);
        if (levelEditor.Length != 0)
            MakeNextLevel();
        GameEvents.instance.onNextLevelEvent += NextLevel;
        GameEvents.instance.onGameRestartEvent += Restart;
    }

    private void FixedUpdate()
    {
        transform.localEulerAngles += new Vector3(0f, (rotationSpeed.Length != 0)?(rotationSpeed[levelIndex] * Time.fixedDeltaTime):(0f), 0f);
    }

    private void MakeNextLevel()
    {
        string level = levelEditor[levelIndex];
        char[] legalChars = { 'o', 'e' };
        string leftover = level;
        int currentUnitIndex = 0;
        int endIndex;
        while(leftover.IndexOfAny(legalChars, 1) != -1)
        {
            endIndex = leftover.IndexOfAny(legalChars, 1);
            string set = leftover.Substring(0, endIndex);
            int count = int.Parse(leftover.Substring(1, endIndex - 1));
            if (set[0] == 'o')
            {
                for (int i = 0; i < count; i++)
                {
                    GameObject childObject = Instantiate(obstacle);
                    obstacles.Add(childObject);
                    childObject.transform.parent = gameObject.transform;
                    childObject.transform.localPosition = new Vector3(0, 0, 0);
                    childObject.transform.localEulerAngles = new Vector3(0f, currentUnitIndex * 360f / numberOfUnits, 0f);
                    currentUnitIndex++;
                }
            }
            else if (set[0] == 'e')
            {
                currentUnitIndex += count;
            }
            leftover = leftover.Substring(endIndex);
        }
        if (leftover[0] == 'o')
        {
            for (int i = 0; i < int.Parse(leftover.Substring(1)); i++)
            {
                GameObject childObject = Instantiate(obstacle);
                obstacles.Add(childObject);
                childObject.transform.parent = gameObject.transform;
                childObject.transform.localPosition = new Vector3(0, 0, 0);
                childObject.transform.localEulerAngles = new Vector3(0f, currentUnitIndex * 360f / numberOfUnits, 0f);
                currentUnitIndex++;
            }
        }
        levelIndex = (levelIndex + 1) % levelEditor.Length;
    }

    private void Restart()
    {
        levelIndex = 0;
        NextLevel();
    }

    private void NextLevel()
    {
        foreach (GameObject element in obstacles)
        {
            //obstacles.Remove(element);
            Destroy(element);
        }
        obstacles = new ArrayList();
        transform.localEulerAngles = new Vector3(0f, initialRotationY + rotationOffset, 0f);
        if (levelEditor.Length != 0)
            MakeNextLevel();
    }
}
